using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using WiimoteLib;
using System;

static class Program
{ static void Main() { Game1 game = new Game1(); game.Run(); }
  public class Game1 : Microsoft.Xna.Framework.Game
  { private GraphicsDeviceManager g;
    private VertexBuffer vbuffer;  
    private BasicEffect  effect;
    private EffectPass   pass;
    private Form1   form = new Form1();
    private Wiimote bb   = new Wiimote();
    const int   cylFaces  = 40;
    const float cylRadius = 3f;
    VertexPositionNormalTexture[] cylinder = new VertexPositionNormalTexture[cylFaces];
    //public static float[] height = new float [Form1.nTrackBars];
    Matrix[] zoom        = new Matrix[6];
    Matrix[] shift       = new Matrix[6];
    Matrix[] shiftToZero = new Matrix[6];
    public static float[] height = { 10f, 10f, 5f, 5f, 30f, 1f };
    public static Vector2 gravity = new Vector2( 10f, 10f );
    public Game1() 
    { g = new GraphicsDeviceManager( this ); 
      this.Exiting += new System.EventHandler( OnExit );
    }
    protected override void Initialize()
    { g.PreferredBackBufferWidth = 500;
      g.PreferredBackBufferHeight = 500;
      g.ApplyChanges();
      g.IsFullScreen = false;
      Window.AllowUserResizing = true;
      Window.Title = "XNA using Wii Balance Bord";
      try { bb.Connect();
            bb.SetLEDs(1);
          }
      catch { System.Windows.Forms.MessageBox.Show( "Can't find a BalanceBoard" ); Exit(); }
      form.Location = new System.Drawing.Point( Window.ClientBounds.Right + 8, 
                                                Window.ClientBounds.Top );
      form.Size = new System.Drawing.Size( 100, Window.ClientBounds.Height );
      form.Show();
      base.Initialize();
      double arcus = MathHelper.TwoPi / cylFaces;
      for ( int i=0; i < cylFaces; i++ )
      { cylinder[i].Normal.X = (float)System.Math.Cos( i * arcus );
        cylinder[i].Normal.Z = (float)System.Math.Sin( i * arcus );
        cylinder[i].Position.X = cylRadius * cylinder[i].Normal.X;
        cylinder[i].Position.Z = cylRadius * cylinder[i].Normal.Z;
        if ( i%2 == 0 ) cylinder[i].Position.Y =  0.5f;
        else            cylinder[i].Position.Y = -0.5f;
      }
      vbuffer = new VertexBuffer( g.GraphicsDevice, typeof(VertexPositionNormalTexture), cylFaces, BufferUsage.WriteOnly );
      vbuffer.SetData< VertexPositionNormalTexture >( cylinder );
      effect = new BasicEffect( g.GraphicsDevice, null );
      //effect.View = Matrix.CreateLookAt( new Vector3(0, -8, 10), new Vector3(0, 0, 0), Vector3.Up );
      effect.View = Matrix.CreateLookAt( new Vector3(3, -30, 100), Vector3.Zero, Vector3.Up );
      //effect.Projection = Matrix.CreatePerspectiveFieldOfView( MathHelper.Pi/4, 1f, 0.1f, 100f );
      effect.Projection = Matrix.CreateOrthographic( 80, 80, 0.01f, 100f );
      effect.LightingEnabled = true;
      //effect.Texture = new ContentManager(Services).Load< Texture2D >("Content\\Foto");
      //effect.TextureEnabled = true;
      pass = effect.CurrentTechnique.Passes[0];
      BasicDirectionalLight light = effect.DirectionalLight0;
      light.Direction = new Vector3( 0f, 0f, 1f ); 
      light.Direction.Normalize();
      light.DiffuseColor  = Color.White.ToVector3();
      light.SpecularColor = Color.CornflowerBlue.ToVector3();
      light.Enabled = true;
    }
    protected override void Update(GameTime gameTime)
    { WiimoteState s = bb.WiimoteState;
      BalanceBoardState   bbs = s.BalanceBoardState;
      BalanceBoardSensorsF kg = bbs.SensorValuesKg;
      height[0] = MathHelper.Max( 1f, kg.TopLeft    /4 );
      height[1] = MathHelper.Max( 1f, kg.TopRight   /4 );
      height[2] = MathHelper.Max( 1f, kg.BottomLeft /4 );
      height[3] = MathHelper.Max( 1f, kg.BottomRight/4 );
      height[4] = MathHelper.Max( 1f, bbs.WeightKg  /4 );
      height[5] = 1f;
      gravity.X =     bbs.CenterOfGravity.X;
      gravity.Y = -3f*bbs.CenterOfGravity.Y;
      for ( int i=0; i < 5; i++ )
        zoom[i] = Matrix.CreateScale( 1f, height[i], 1f );
      zoom[5] = Matrix.CreateScale( 0.5f, 0.5f, 0.5f );
      shift[0] = Matrix.CreateTranslation( -20f, height[0]/2f-20, 80f );
      shift[1] = Matrix.CreateTranslation(  20f, height[1]/2f-20, 80f );
      shift[2] = Matrix.CreateTranslation( -20f, height[2]/2f-20, 20f );
      shift[3] = Matrix.CreateTranslation(  20f, height[3]/2f-20, 20f );
      shift[4] = Matrix.CreateTranslation(   0f, height[4]/2f-20, 50f );
      shift[5] = Matrix.CreateTranslation(  gravity.X, height[5]/2f-20, gravity.Y + 50f );
      for ( int i=0; i < 5; i++ ) 
      { try { form.trackbar[i].Value = (int)height[i]; } 
        catch { form.trackbar[i].Value = 150; }
      }
      try { form.trackbar[5].Value = Convert.ToInt32( gravity.X );
            form.trackbar[6].Value = Convert.ToInt32( gravity.Y );
          } catch {}
      g.GraphicsDevice.Vertices[0].SetSource( vbuffer, 0, VertexPositionNormalTexture.SizeInBytes );
      g.GraphicsDevice.VertexDeclaration = new VertexDeclaration( g.GraphicsDevice, VertexPositionNormalTexture.VertexElements );
      g.GraphicsDevice.RenderState.CullMode = CullMode.None;
      //g.GraphicsDevice.RenderState.FillMode = FillMode.WireFrame;
      g.GraphicsDevice.RenderState.DepthBufferEnable      = false;
      g.GraphicsDevice.RenderState.DepthBufferWriteEnable = false;
    }
    protected override void Draw( GameTime gameTime )
    { g.GraphicsDevice.Clear( Color.DarkBlue );
      effect.Begin();
      for ( int i = 0; i < 6; i++ )
      { pass.Begin();
          effect.World =  zoom[i] * shift[i];
          try { g.GraphicsDevice.DrawPrimitives( PrimitiveType.TriangleStrip, 0, cylFaces-2 ); } catch {}
        pass.End();
      }
      effect.End();
    }    
    private void OnExit( object sender, EventArgs e )
    { bb.Disconnect();
      form.Close();
    }
  } // end of class Game1
} // end of class Program